from Crypto import Random
from Crypto.Cipher import AES
import hashlib
import base64


class EpsumCryptAES:
    def encrypt(self, plaintext, key_):
        """Encrypts the plaintext of str type using AES-256 bit CBC specification

        Arguments:
        plaintext -- Plain text to be encrpted
        key_      -- Key for the AES Encryption
        """
        key = hashlib.sha256(key_.encode('utf-8')).digest()
        iv = Random.new().read(AES.block_size)  # iv is of 16bytes block size
        cipher = AES.new(key, AES.MODE_CBC, iv)
        encrypted_text = (cipher.encrypt(bytes(self.pad(plaintext), encoding="utf-8")))
        byte64_encoded = base64.b64encode(iv + encrypted_text)
        return str(byte64_encoded, encoding='utf-8')

    def decrypt(self, cipher, key_, applySHA=False):
        key = key_
        if applySHA:
            key = hashlib.sha256(key_.encode('utf-8')).digest()

        cipher = base64.b64decode(bytes(cipher, 'utf-8'))
        iv = cipher[:16]
        cipher_dec = AES.new(key, AES.MODE_CBC, iv)
        decrypted = cipher_dec.decrypt(cipher[16:])
        decrypted = str(self.unpad(decrypted), encoding="utf-8")
        if len(decrypted) > 0:
            return decrypted
        else:
            raise Exception("Decryption failed")

    def getSHA256Bytes(self, data):
        return hashlib.sha256(data.encode('utf-8')).digest()

    def pad(self, s):
        BS = 16
        return s + (BS - len(s) % BS) * chr(BS - len(s) % BS)

    # unpad = lambda s : s[0:-s[-1]]

    def unpad(self, s):

        return s[0:-s[-1]]



import json as j
from homeautomationcli.utils import *
from homeautomationcli.encryption import *
import sys


# User Login
def login(useremail, userpass):
    aes = EpsumCryptAES()
    Authorization = aes.encrypt('{"message":"hello"}', userpass)
    header = {'USER': useremail, 'Authorization': Authorization}
    Getrequest('/user/login', header)


# User Registration
def user_regd():
    aes = EpsumCryptAES()
    print("Email ID :")
    email = sys.stdin.readline()
    email = email[:-1]

    print("Your Name :")
    name = sys.stdin.readline()
    name = name[:-1]

    print("Date Of Birth :")
    dob = sys.stdin.readline()
    dob = dob[:-1]

    print("Mobile :")
    phone = sys.stdin.readline()
    phone = phone[:-1]

    print("Nationality :")
    nationality = sys.stdin.readline()
    nationality = nationality[:-1]

    print("Gender :")
    gender = sys.stdin.readline()
    gender = gender[:-1]

    print("Password :")
    password = sys.stdin.readline()
    password = password[:-1]

    key = j.loads(regd_key(showOutput=False))["secret"]
    key = str(base64.b64decode(key), encoding="utf-8")
    body = j.dumps(
        {"name": name, "dob": dob, "phone": phone, "nationality": nationality, "password": password, "gender": gender})

    body = aes.encrypt(body, key)
    header = {'User': email}
    Putrequest('/user', header, body)


# View user profile
def profile(useremail, userpass):
    aes = EpsumCryptAES()
    Authorization = aes.encrypt('{"message":"hello"}', userpass)
    header = {'USER': useremail, 'Authorization': Authorization}
    Getrequest('/user', header)


# User Profile Edit
def profile_edit(useremail, userpass):
    aes = EpsumCryptAES()
    print("Address :")
    address = sys.stdin.readline()
    address = address[:-1]

    print("Phone :")
    phone = sys.stdin.readline()
    phone = phone[:-1]

    print("Social")
    social = sys.stdin.readline()
    social = social[:-1]
    social = j.loads(social)

    print("ID Proof")
    idproof = sys.stdin.readline()
    idproof = idproof[:-1]
    idproof = j.loads(idproof)

    body = j.dumps({"address": address, "phone": phone, "social": social, "idproof": idproof})
    body = aes.encrypt(body, userpass)
    header = {'USER': useremail}
    Postrequest('/user', header, body)


# Get User Setting
def user_setting(useremail, userpass):
    aes = EpsumCryptAES()
    Authorization = aes.encrypt('{}', userpass)
    header = {"USER": useremail, "Authorization": Authorization}
    Getrequest('/user/settings', header)


# Update User Setting
def user_setting_update(useremail, userpass):
    aes = EpsumCryptAES()
    print("Home")
    home = sys.stdin.readline()
    home = home[:-1]
    home = j.loads(home)

    print("Rooms")
    rooms = sys.stdin.readline()
    rooms = rooms[:-1]
    rooms = j.loads(rooms)

    body = j.dumps({"homes": home, "rooms": rooms})
    body = aes.encrypt(body, userpass)
    header = {"USER": useremail}
    Postrequest('/user/settings', header, body)


# Control modules
def control_module(useremail, userpass):
    aes = EpsumCryptAES()
    print("Thing ID :")
    thingid = sys.stdin.readline()
    thingid = thingid[:-1]

    print("Congiguration (json[])")
    config = sys.stdin.readline()
    config = config[:-1]
    config = j.loads(config)

    body = j.dumps({"thingid": thingid, "config": config})
    body = aes.encrypt(body, userpass)
    header = {"USER": useremail}
    Postrequest('/thing', header, body)


# Module Info
def module_info(useremail, userpass):
    aes = EpsumCryptAES()
    print("Thing ID :")
    thingid = sys.stdin.readline()
    thingid = thingid[:-1]
    Authorization = j.dumps({"message": "hello", "thingid": thingid})
    header = {"USER": useremail, "Authorization": Authorization}
    Getrequest('/thing', header)


# Module Registration
def module_regd(useremail, userpass):
    aes = EpsumCryptAES()

    print("Thing Id :")
    thingid = sys.stdin.readline()
    thingid = thingid[:-1]

    print("Thing Name :")
    thingname = sys.stdin.readline()
    thingname = thingname[:-1]

    header = {"USER": useremail}
    body = j.dumps({"thingid": thingid, "thingname": thingname})
    body = aes.encrypt(body, userpass)
    Putrequest('/thing/register', header, body)


# Module Configure
def module_config(useremail, userpass):
    aes = EpsumCryptAES()
    print("Thing Name :")
    thingname = sys.stdin.readline()
    thingname = thingname[:-1]
    body = j.dumps({"thingname": thingname})
    body = aes.encrypt(body, userpass)
    header = {"USER": useremail}
    Postrequest('/thing/config', header, body)


# Forgot Password Step-1
def forgot_1(useremail, userpass):
    aes = EpsumCryptAES()
    print("Token")
    token = sys.stdin.readline()
    token = token[:-1]
    body = j.dumps({"token": token})
    body = aes.encrypt(body, userpass)
    header = {"USER": useremail}
    Postrequest('/user/forgot1', header, body)


# Forgot Password Step-2
def forgot_2(useremail, userpass):
    aes = EpsumCryptAES()
    print("Token")
    token = sys.stdin.readline()
    token = token[:-1]

    print("New Password")
    newpassword = sys.stdin.readline()
    newpassword = newpassword[:-1]

    body = j.dumps({"newpassword": newpassword, "token": token})
    body = aes.encrypt(body)
    header = {"USER": useremail}
    Postrequest('/user/forgot2', header, body)


# Update User Password
def password_update(useremail, userpass):
    aes = EpsumCryptAES()
    print("Password")
    password = sys.stdin.readline()
    password = password[:-1]
    body = j.dumps({"password": password})
    body = aes.encrypt(body)
    header = {"USER": useremail}
    Postrequest('/user/update', header, body)


# Show History
def history(useremail, userpass):
    aes = EpsumCryptAES()
    Authorization = j.dumps({"message": "hello"})
    Authorization = aes.encrypt(Authorization, userpass)
    header = {"USER": useremail}
    Getrequest('/user/activity', header)


# Get Key For Registration
def regd_key(showOutput=True):
    return Getrequest('/getkey', '', showOutput)


# Admin actions

# Add a thing definition
def thingreg_from_base_model():
    print('Enter admin email : ')
    email = sys.stdin.readline()[:-1]
    print('Password : ')
    password = sys.stdin.readline()[:-1]
    print("New Thing ID : ")
    thingid = sys.stdin.readline()[:-1]
    print("Thing name: ")
    thingname = sys.stdin.readline()[:-1]
    print("Base Model: ")
    basemodel = sys.stdin.readline()[:-1]
    header = {"USER": email}
    body = j.dumps({"thingid": thingid, "thingname": thingname, 'basemodel': basemodel})
    aes = EpsumCryptAES()
    body = aes.encrypt(body, password)
    Postrequest('/thing/register', header, body)


def thingDefinition():
    print('Enter admin email : ')
    email = sys.stdin.readline()[:-1]
    print('Password : ')
    password = sys.stdin.readline()[:-1]
    print("Model name : ")
    modelname = sys.stdin.readline()[:-1]
    print("Thing Type: ")
    thingtype = sys.stdin.readline()[:-1]
    print("Schema: ")
    schema = sys.stdin.readline()[:-1]
    header = {"USER": email}
    body = j.dumps({"modelname": modelname, "thingtype": thingtype, 'schema': j.loads(schema)})
    aes = EpsumCryptAES()
    body = aes.encrypt(body, password)
    Putrequest('/thing/define', header, body)


def adminActions():
    print("1- Add a Thing defination")
    print("2- Register a thing with a unique id")
    print("3- Back")
    choice = sys.stdin.readline()
    choice = int(choice[:-1])
    if choice == 1:
        thingDefinition()
    elif choice == 2:
        thingreg_from_base_model()
    elif choice == 3:
        from .commandline import entry
        entry()
    else:
        print("Invalid Choice.. Try again")
        adminActions()


from distutils.core import setup

long_description='''
	This project is a command line utility tool, complimented with Epsumlabs Open Source IoT platform project.
'''
setup(
    name='homeautomationcli',
    version='0.0.2',
    packages=['homeautomationcli'],
    url='https://github.com/Epsumlabs/homeautomationcli',
    license='MIT',
    author='saiprasad',
    entry_points = {'console_scripts':['homeautomationcli=homeautomationcli.command_line:main'],},
    scripts=['homeautomationcli/commandline.py'],
    author_email='saiprasad1996@hotmail.com',
    install_requires=[
          'pycrypto',
          'requests'
      ],
    description='Epsum Labs Home Automation CLI',
    long_description=long_description,
    keywords = ['cli', 'homeautomation', 'IoT'],
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5'
    )
)



import requests

baseurl = "http://192.168.1.112:8080/api"


# Get Request Function
def Getrequest(endpoint, header, showOutput=True):
    url = baseurl + endpoint
    response = requests.get(url=url, headers=header)
    if showOutput:
        print("Output:")
        print(response.json())
    return response.text


# Post Request Function
def Postrequest(endpoint, header, body):
    url = baseurl + endpoint
    response = requests.post(url=url, headers=header, data=body)
    print("Output:")
    print(response.json())


# Put Request Function
def Putrequest(endpoint, header, body):
    url = baseurl + endpoint
    response = requests.put(url=url, headers=header, data=body)
    print("Output:")
    print(response.json())



from homeautomationcli.definations import (login, user_regd, profile, profile_edit, user_setting, user_setting_update,
                                           control_module,
                                           module_info, module_regd, module_config, forgot_1, forgot_2, password_update,
                                           history,
                                           regd_key, adminActions)
import sys


# Main Function
def entry():
    try:
        print("\n----------------------------------------------------")
        print("\n\nEpsum labs Home Automation Console App")
        print("Developed by Epsum labs Pvt. Ltd.")
        print("Authors : Suraj Kumar, Sai Prasad")
        print("Released Under MIT License")
        print("\n----------------------------------------------------")
        print("1- Login")
        print("Press any key to Register")
        choice = sys.stdin.readline()[:-1]
        assert choice == str(1)
        print("Enter your registered email with Epsumthings account : ")
        useremail = sys.stdin.readline()
        useremail = useremail[:-1]
        print("Enter you account password : ")
        userpass = sys.stdin.readline()
        userpass = userpass[:-1]
        print("\n")
        if useremail == "" or userpass == "":
            print("Please provide your username and password to login ...")
            entry()
        while (True):
            print("----------------------------------------------------")
            print("Logged In as : " + str(useremail) + "\n")
            print()
            print("Choose an option from the following menu : \n")
            print("1. Login ")
            print("2. User Registration")
            print("3. User Profile Display")
            print("4. User profile edit")
            print("5. Get User settings")
            print("6. Update user settings")
            print("7. Control modules")
            print("8. Module Info")
            print("9. Module registration")
            print("10. Module Configure")
            print("11. Forgot password step-1")
            print("12. Forgot password step-2")
            print("13. Update user password")
            print("14. Show History")
            print("15. Get Key for register")
            print("16. Admin Actions")
            print("0. Exit")
            choice = sys.stdin.readline()
            choice = int(choice[:-1])
            if choice == 1:
                login(useremail, userpass)
            elif choice == 2:
                user_regd()
            elif choice == 3:
                profile(useremail, userpass)
            elif choice == 4:
                profile_edit(useremail, userpass)
            elif choice == 5:
                user_setting(useremail, userpass)
            elif choice == 6:
                user_setting_update(useremail, userpass)
            elif choice == 7:
                control_module(useremail, userpass)
            elif choice == 8:
                module_info(useremail, userpass)
            elif choice == 9:
                module_regd(useremail, userpass)
            elif choice == 10:
                module_config(useremail, userpass)
            elif choice == 11:
                forgot_1(useremail, userpass)
            elif choice == 12:
                forgot_2(useremail, userpass)
            elif choice == 13:
                password_update(useremail, userpass)
            elif choice == 14:
                history(useremail, userpass)
            elif choice == 15:
                regd_key()
            elif choice == 16:
                adminActions()
            elif choice == 0:
                sys.exit()
            else:
                print("Invalid Choice")
            print("Press enter any key to continue")
            sys.stdin.readline()
    except KeyError:
        print(" You entered a wrong/invalid input.. Please enter a number from the menu... ")
        entry()
    except AssertionError:
        user_regd()
        entry()
    except KeyboardInterrupt:
        print("Exiting..")
        sys.exit(0)
    except ValueError:
        entry()
